/*
 *  server_ip is the IP address of the ESP8266 module, will be 
 *  printed to Serial when the module is connected.
 */

#include <ESP8266WiFi.h>

const char* ssid = "Z21_6666";
const char* password = "54448049";

// Create an instance of the server
// specify the port to listen on as an argument 80
WiFiServer server(4711);

boolean alreadyConnected = false; // whether or not the client was connected previously

// Параметры декодера для настройки:
#define DECODER_ADDRESS_1  22    // Адрес декодера 1
#define DECODER_ADDRESS_2  23    // Адрес декодера 2

// Управляющие пины, где wire номер = номеру провода на схеме светофора Viessmann H0 Licht-Ausfahrsignal Art.Nr  4013:
#define WIRE1_PIN  4  // D2
#define WIRE2_PIN  5  // D1
#define WIRE3_PIN  14 // D5
#define WIRE4_PIN  12 // D6
// #define WIRE5_PIN  13 // D7

   // FB part:
//--------------------------------------------------------------------------------------------
 boolean flag0 = false;    // метка о том, что препятствие уже посчитали
 byte last_data = 0; 
 byte data = 0;            // выходное значение

// Обработчик стрелочных пакетов. Эта функция запускается, когда библиотека ловит пакет:
void switchSend(byte address, byte data)
{
// Serial.print("Z , ");
// Serial.print(address);
// Serial.print(", ");
// Serial.println(data);
// Обрабатываем адрес стрелки:
 if (address == DECODER_ADDRESS_1) {// Если адрес в пакете == адресу декодера, то обрабатываем.
  switch (data) {
   case 0x00: 
    digitalWrite(WIRE1_PIN, LOW); digitalWrite(WIRE2_PIN, HIGH); digitalWrite(WIRE3_PIN, HIGH); digitalWrite(WIRE4_PIN, HIGH);// digitalWrite(WIRE5_PIN, HIGH);
    break;
   case 0x01:
    digitalWrite(WIRE1_PIN, HIGH); digitalWrite(WIRE2_PIN, LOW); digitalWrite(WIRE3_PIN, HIGH); digitalWrite(WIRE4_PIN, HIGH);// digitalWrite(WIRE5_PIN, HIGH);
    break;
   default:   // UNKNOWN_COMMAND 
    // Serial.print("UNKNOWN_COMMAND 0x"); 
    // Serial.println(data, HEX);
    break;
  }
 }

 if (address == DECODER_ADDRESS_2) {// Если адрес в пакете == адресу декодера, то обрабатываем.
  switch (data) {    
   case 0x00:
    digitalWrite(WIRE1_PIN, HIGH); digitalWrite(WIRE2_PIN, HIGH); digitalWrite(WIRE3_PIN, LOW); digitalWrite(WIRE4_PIN, HIGH);// digitalWrite(WIRE5_PIN, HIGH);
    break;
   case 0x01:
    digitalWrite(WIRE1_PIN, HIGH); digitalWrite(WIRE2_PIN, HIGH); digitalWrite(WIRE3_PIN, HIGH); digitalWrite(WIRE4_PIN, LOW);// digitalWrite(WIRE5_PIN, LOW);
    break;
   default:   // UNKNOWN_COMMAND 
    // Serial.print("UNKNOWN_COMMAND 0x"); 
    // Serial.println(data, HEX);
    break;
  }
 }
}// end of handler

void setup() {
  Serial.begin(115200);
  delay(10);

  // Connect to WiFi network
  Serial.println();
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);
  
  WiFi.begin(ssid, password);
  
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.println("WiFi connected");
  
  // Start the server
  server.begin();
  Serial.println("Server started");

  // Print the IP address
  Serial.println(WiFi.localIP());

  //Выставляем управляющие пины на вывод:
  pinMode(WIRE1_PIN, OUTPUT); pinMode(WIRE2_PIN, OUTPUT); pinMode(WIRE3_PIN, OUTPUT); pinMode(WIRE4_PIN, OUTPUT);// pinMode(WIRE5_PIN, OUTPUT);

  //Выставляем их в HIGН, что бы при запуске светофор не горел. Так как они подключены к общему плюсу и минусом к выводам, то управляющая логика будет обратной:
  // HIGH = OFF, LOW = ON:
  digitalWrite(WIRE1_PIN, LOW); digitalWrite(WIRE2_PIN, LOW); digitalWrite(WIRE3_PIN, HIGH); digitalWrite(WIRE4_PIN, HIGH);// digitalWrite(WIRE5_PIN, HIGH);
   
}

void loop() {
  // Check if a client has connected
  WiFiClient client = server.available();
  if (!client) {
    return;
  }
 
  Serial.println("new client");  // Wait until the client sends some data
  
  while(!client.available()){

// датчик 0
  if ((!digitalRead(13)) && (flag0 == false)){    // сравниваем. Если меньше И препятствие еще не зафиксировано
     data = (data + 1);                          // 
     flag0 = true;                               // ставим метку, что препятствие посчитано
     }
  if ((digitalRead(13)) && (flag0 == true)){     // опять сравниваем. Если больше И препятствие уже было зафиксировано
     data = (data - 1);
     flag0 = false;                              // сбрасываем метку, т.е. следующее препятствие будет снова посчитано
     }

  if (data != last_data){    //Если состояние входа изменилось, расскажем
    if (data == 1){ 
      client.println("<fb id=\"fb-5\" state=\"true\" fbtype=\"0\"/>");
    } else { 
      client.println("<fb id=\"fb-5\" state=\"false\" fbtype=\"0\"/>");
    }
    last_data = data; // запоминаем текущие состояния
    }

    delay(1);
  }

  //--- chtenie stroki
 String req = client.readStringUntil('/');

 client.stop();
 Serial.println("client disonnected");
    
 Serial.println(req);

 //--- obrabotka
 byte address = ((req[29]-48)*10)+(req[30]-48);
 byte data = req[48] - 48;
 switchSend( address, data);

} //END loop

//End of sketch

